/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  curl.h
 *   Simple wrapper around libCurl to download a remote URI simply.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#ifndef _CURL_H
#define _CURL_H

#include <curl/curl.h>

/**
 * This is a simple wrapper around the libCurl library, which allows the
 * user to download the contents of an arbitary URI via a LUA script.
 *
 * Sample usage looks like this:
 *
 * @code
 *
 *   CCurl fetcher;
 *   fetcher.fetchURI( "http://localhost/" );
 *  
 *   data          = fetcher.getData();
 *   HTTP_Response = fetcher.getCode();
 *
 * @endcode
 *
 * @version $Id: curl.h,v 1.2 2005/10/08 14:43:23 steve Exp $
 * @author Steve Kemp http://www.steve.org.uk
 */
class CCurl
{

public:

    /**
     * Constructor.
     * @returns A new CCurl object.
     */
    CCurl( );


    /**
     * Destructor.  Tidy up and free memory.
     */
    ~CCurl();


public:


    /**
     * Download the named URI object.
     * @param uri The URI of the resource to download.
     */
    void fetchURI( const char *uri );

    
    /**
     * Get the HTTP status code for the previous transaction.
     * @return The HTTP Status code of the previous download, via fetchURI
     * @see fetchURI
     */
    long getCode();
    

    /**
     * get the data from the page/object.
     * @return The contents of the URI we download, via fetchURI
     * @see fetchURI
     */
    char *getData();


protected:


    /**
     * Callback routine for libCurl.
     * @param ptr A pointer to the data chunk downloaded.
     * @param size The size of data we are being passed.
     * @param nmemb The size of each byte of data included in size.
     * @param userp A user-supplied pointer.
     * @return The amount of data successfully managed (and added to our m_data buffer).
     */
    static size_t write_data(void *ptr, size_t size, size_t nmemb, void *userp);


private:


    /**
     * Curl object.
     */
    CURL *m_url;


    /**
     * Data returned from a successful page fetch.
     */
    void *m_data;


    /**
     * Size of the date we have returned.
     */
    int m_data_size;


};

#endif /* _CURL_H */
