/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/*
 *  luainstance.cpp
 *   Singleton wrapper around a LUA intepretter.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include "luainstance.h"
#include "curl.h"
#include "services.h"
#include "servicedetect.h"
#include "md5.h"
#include "sha1.h"
#include "portscanner.h"

#include "version.h"
InstallVersion dummyLUA( __FILE__, __DATE__, "$Revision: 1.11 $" );



/*
 * Set our instance pointer to NULL - this will ensure the
 * singleton accessor works as expected.
 */
CLUAInstance * CLUAInstance::m_instance = NULL;




/**
 * Constructor.
 *
 * This isn't public as we're a singleton.
 */
CLUAInstance::CLUAInstance()
{
    /* Create the intepretter object.  */
    m_lua =  lua_open();

    /* Register the standard functions. */
    registerFunctions();
}


/**
 * Destructor.
 */
CLUAInstance::~CLUAInstance()
{
    if ( m_lua != NULL )
    {
        lua_close( m_lua );
	m_lua = NULL;
    }
}


/**
 * Singleton accessor to this class.
 */
/* static */ CLUAInstance *CLUAInstance::getInstance()
{
    if ( !m_instance )
    {
	m_instance = new CLUAInstance( );
    }

    return( m_instance );
}



/**
 * Run the given script.
 * TODO: Test file exists.
 */
void CLUAInstance::runScript( std::string fileName )
{
    /* run the script */
    lua_dofile( m_lua, fileName.c_str() );
}


/**
 * Call a LUA function from C++.
 *
 * @param func The name of the function to call.
 * @param sig A format string of input and output options with a ">" between them.
 *
 * For example:
 * 
 *  callFunction("f", "dd>d", x, y, &z);
 *
 * Calls a function with two "double" arguments, returning a double.
 *
 * @seealso callFunctionError
 * @author http://www.lua.org/pil/25.3.html
 */
void CLUAInstance::callFunction(const char *func, const char *sig, ...) 
{
    va_list vl;
    int narg, nres;  /* number of arguments and results */
    
    va_start(vl, sig);
    lua_getglobal( m_lua, func );  /* get function */
    
    /* push arguments */
    narg = 0;
    while (*sig) 
    {  
        /* push arguments */
        switch (*sig++) 
	{
    
          case 'd':  /* double argument */
            lua_pushnumber( m_lua, va_arg(vl, double));
            break;
    
          case 'i':  /* int argument */
            lua_pushnumber( m_lua, va_arg(vl, int));
            break;
    
          case 's':  /* string argument */
            lua_pushstring( m_lua, va_arg(vl, char *));
            break;
    
          case '>':
            goto endwhile;
    
          default:
            callFunctionError("invalid option (%c)", *(sig - 1));
        }
        narg++;
        luaL_checkstack( m_lua, 1, "too many arguments");
    } 
    endwhile:
    

    /* do the call */
    nres = strlen(sig);  /* number of expected results */
    if (lua_pcall( m_lua, narg, nres, 0) != 0)  /* do the call */
      callFunctionError("error running function `%s': %s", func, lua_tostring( m_lua, -1));
    
    /* retrieve results */
    nres = -nres;  /* stack index of first result */
    while (*sig) 
    {  
        /* get results */
        switch (*sig++) 
	{
    
	  case 'd':  /* double result */
            if (!lua_isnumber( m_lua, nres))
              callFunctionError( "wrong result type");
            *va_arg(vl, double *) = lua_tonumber(m_lua, nres);
            break;
    
	  case 'i':  /* int result */
	    if (!lua_isnumber( m_lua, nres))
              callFunctionError("wrong result type");
            *va_arg(vl, int *) = (int)lua_tonumber(m_lua, nres);
            break;
    
          case 's':  /* string result */
            if (!lua_isstring( m_lua, nres))
              callFunctionError("wrong result type");
            *va_arg(vl, const char **) = lua_tostring(m_lua, nres);
            break;
    
          default:
            callFunctionError("invalid option (%c)", *(sig - 1));
        }
        nres++;
    }
    va_end(vl);
}


/**
 * Called if "callFunction" returns an error.
 * TODO: This should be a callback, or in a derived class..
 * @seealso callFunction
 */
void CLUAInstance::callFunctionError( const char *fmt, ...) 
{
    va_list argp;
    va_start(argp, fmt);
    vfprintf(stderr,fmt, argp);
    va_end(argp);
    lua_close(m_lua);
    exit(EXIT_FAILURE);
}

/**
 * Load the LUA basic functions and register some C callbacks of our own.
 */
void CLUAInstance::registerFunctions()
{
    /*
     * Ensure we have the full LUA basic functions available to us.
     */
    lua_baselibopen(m_lua);
    luaopen_table(m_lua);
    luaopen_io(m_lua);
    luaopen_string(m_lua);
    luaopen_math(m_lua);
    

    /*
     * Register our extra functions.
     *
     * TODO: This should really be a virtual function in a derived class.
     */
    lua_register(m_lua, "get_uri",                LUAGetURI        );
    lua_register(m_lua, "service_name_from_port", LUAServiceByPort );
    lua_register(m_lua, "service_port_from_name", LUAServiceByName );
    lua_register(m_lua, "detect_service",         LUADetectService );
    lua_register(m_lua, "md5_string",             LUAMD5Sum        );
    lua_register(m_lua, "md5_file",               LUAMD5SumFile    );
    lua_register(m_lua, "sha1_string",            LUASHA1Sum       );
    lua_register(m_lua, "sha1_file",              LUASHA1SumFile   );
    lua_register(m_lua, "portscan",               LUAOpenPorts     );
    lua_register(m_lua, "to_ip",                  LUAToIP          );
    lua_register(m_lua, "to_host",                LUAToHost        );

    /*
     * Register our version number in the global variable VERSION.
     */
    lua_pushstring(m_lua, "VERSION");
    lua_pushstring(m_lua, VERSION);
    lua_settable(m_lua, LUA_GLOBALSINDEX);
}




/**
 * A 'curl' wrapper for LUA scripts.
 */
/* static */ int CLUAInstance::LUAGetURI( lua_State *L )
{
    /* get number of arguments */
    int n = lua_gettop(L);
	
    if ( n != 1 )
    {
        lua_pushstring(L, "One argument is required for 'get_uri'");
	lua_error(L);
    }

    if (!lua_isstring(L, 1)) 
    {
        lua_pushstring(L, "String argument required for 'get_uri'");
	lua_error(L);
    }

    
    /*
     * Get the argument.
     */
    char *uri  = (char *)lua_tostring( L, 1 );
    
    /* Results */
    long ret   = -1;
    char *data = NULL;
    
    /* Fetch the URI */
    CCurl fetcher;
    fetcher.fetchURI( uri );
    
    /* Get the results. */
    ret  = fetcher.getCode();
    data = fetcher.getData();
    
    /* Push the results to LUA's stack */
    lua_pushnumber(L, ret );
    lua_pushstring(L, data );

    /* return the number of results */
    return 2;
}



/**
 * A wrapper for LUA scripts.
 */
/* static */ int CLUAInstance::LUAServiceByPort( lua_State *L )
{
    /* get number of arguments */
    int n = lua_gettop(L);
    
    const char *proto = "tcp";

    if ( ( n < 1 ) ||
	 ( n > 2 ) )
    {
        lua_pushstring(L, "Either one or two arguments are required for 'service_by_port'");
	lua_error(L);
    }

    if (!lua_isnumber(L, 1)) 
    {
        lua_pushstring(L, "The first argument to 'service_by_port' must be a port number" );
	lua_error(L);
    }

    if (( n == 2 ) && (!lua_isstring(L, 2)))
    {
        lua_pushstring(L, "The second argument to 'service_by_port' must be a string" );
	lua_error(L);
    }

    
    /*
     * Get the argument.
     */
    char *p = (char *)proto;
    if ( n == 2 )
    {
	p = (char *)lua_tostring( L, 2 );
    }
    
    int port = (int)lua_tonumber( L, 1 );

    /*
     * Lookup the name.
     */
    std::string name = CServices::LookupByPort( port, (const char *) p );

    /* Save the result */
    lua_pushstring( L, name.c_str() );

    /* return the number of results */
    return 1;
}



/**
 * A wrapper for LUA scripts.
 */
/* static */ int CLUAInstance::LUAServiceByName( lua_State *L )
{
    /* get number of arguments */
    int n = lua_gettop(L);
    
    const char *proto = "tcp";

    if ( ( n < 1 ) ||
	 ( n > 2 ) )
    {
        lua_pushstring(L, "Either one or two arguments are required for 'service_by_name'");
	lua_error(L);
    }

    if (!lua_isstring(L, 1)) 
    {
        lua_pushstring(L, "The first argument to 'service_by_name' must be a string" );
	lua_error(L);
    }

    if (( n == 2 ) && (!lua_isstring(L, 2)))
    {
        lua_pushstring(L, "The second argument to 'service_by_name' must be a string" );
	lua_error(L);
    }

    
    /*
     * Get the optional protocol argument - if not present we use 'tcp'.
     */
    char *p = (char *)proto;
    if ( n == 2 )
    {
	p = (char *)lua_tostring( L, 2 );
    }
    
    /*
     * Service name to lookup.
     */
    const char *name = (const char *)lua_tostring( L, 1 );

    /*
     * Lookup the name.
     */
    int result = CServices::LookupByName( name, (const char *) p );

    /* Save the result */
    lua_pushnumber( L, result );

    /* return the number of results */
    return 1;
}



/**
 * A wrapper for LUA scripts.
 */
/* static */ int CLUAInstance::LUADetectService( lua_State *L )
{
    /* get number of arguments */
    int n    = lua_gettop(L);

    if ( n != 2 )
    {
        lua_pushstring(L, "Two arguments are required for 'detect_service'");
	lua_error(L);
    }

    if (!lua_isstring(L, 1)) 
    {
        lua_pushstring(L, "The first argument to 'detect_service' must be a string" );
	lua_error(L);
    }

    if (!lua_isnumber(L, 2))
    {
        lua_pushstring(L, "The second argument to 'detect_service' must be a port number" );
	lua_error(L);
    }

    /*
     * Service name to lookup.
     */
    char *name = (char *)lua_tostring( L, 1 );
    int   port = (int)lua_tonumber( L, 2 );

    /*
     * Lookup the name.
     */
    CServiceDetect guess(  name, port );

    char *rest = guess.getServiceName();


    /* Save the result */
    lua_pushstring( L, rest );

    /* return the number of results */
    return 1;
}



/**
 * C++ function to MD5 checksum a string
 */
/* static */ int CLUAInstance::LUAMD5Sum( lua_State *L )
{
    /* get number of arguments */
    int n    = lua_gettop(L);

    if ( n != 1 )
    {
        lua_pushstring(L, "Only one argument is required for 'md5_string'");
	lua_error(L);
    }

    if (!lua_isstring(L, 1)) 
    {
        lua_pushstring(L, "The first argument to 'md5_string' must be a string" );
	lua_error(L);
    }


    const char *data = lua_tostring( L, 1 );

    CMD5 sum;
    sum.setPlainText( data );

    /* Save the result */
    lua_pushstring( L, sum.getMD5Digest() );

    /* return the number of results */
    return 1;
}


/**
 * C++ function to MD5 checksum a file's contents.
 */
/* static */ int CLUAInstance::LUAMD5SumFile( lua_State *L )
{
    /* get number of arguments */
    int n    = lua_gettop(L);

    if ( n != 1 )
    {
        lua_pushstring(L, "Only one argument is required for 'md5_file'");
	lua_error(L);
    }

    if (!lua_isstring(L, 1)) 
    {
        lua_pushstring(L, "The first argument to 'md5_file' must be a string" );
	lua_error(L);
    }


    const char *fileName = lua_tostring( L, 1 );
    const char *data     = CLUAInstance::readCompleteFile( fileName );

    if ( data == NULL )
    {
	lua_pushstring(L, "The file couldn't be read" );
	lua_error(L);
    }

    CMD5 sum;
    sum.setPlainText( data );

    /* Save the result */
    lua_pushstring( L, sum.getMD5Digest() );

    /* Free file contents */
    free( (void *)data );

    /* return the number of results */
    return 1;
}


/**
 * C++ function to SHA1 checksum a file's contents
 */
/* static */ int CLUAInstance::LUASHA1SumFile( lua_State *L )
{
    /* get number of arguments */
    int n    = lua_gettop(L);

    if ( n != 1 )
    {
        lua_pushstring(L, "Only one argument is required for 'sha1_file'");
	lua_error(L);
    }

    if (!lua_isstring(L, 1)) 
    {
        lua_pushstring(L, "The first argument to 'sha1_file' must be a string" );
	lua_error(L);
    }


    const char *fileName = lua_tostring( L, 1 );

    CSHA1 sha;
    sha.HashFile((char*) fileName );
    sha.Final();

    char result[100];
    memset( result, '\0', sizeof(result));
    sha.ReportHash( result, CSHA1::REPORT_HEX );

    /* Save the result */
    lua_pushstring( L, result );

    /* return the number of results */
    return 1;
}


/**
 * C++ function to MD5 checksum a string
 */
/* static */ int CLUAInstance::LUASHA1Sum( lua_State *L )
{
    /* get number of arguments */
    int n    = lua_gettop(L);

    if ( n != 1 )
    {
        lua_pushstring(L, "Only one argument is required for 'sha1_string'");
	lua_error(L);
    }

    if (!lua_isstring(L, 1)) 
    {
        lua_pushstring(L, "The first argument to 'sha1_string' must be a string" );
	lua_error(L);
    }


    const char *data = lua_tostring( L, 1 );

    CSHA1 sha;
    sha.Update( (unsigned char*)data, strlen( data ) );
    sha.Final();

    char result[100];
    memset( result, '\0', sizeof(result));
    sha.ReportHash( result, CSHA1::REPORT_HEX );

    /* Save the result */
    lua_pushstring( L, result );

    /* return the number of results */
    return 1;
}



/**
 * C++ function to return the open ports upon a host.
 */
/* static */ int CLUAInstance::LUAOpenPorts( lua_State *L )
{
    /* get number of arguments */
    int n         = lua_gettop(L);
    char *host    = NULL;

    if ( n == 0  )
    {
	host = "localhost" ;
    }

    if ( n > 1 )
    {
	lua_pushstring(L, "No more than one argument is required for 'portscan'");
	lua_error(L);
    }

    if ( n == 1 && (!lua_isstring(L, 1) ) )
    {
        lua_pushstring(L, "The argument to 'portscan' must be a string" );
	lua_error(L);
    }
    

    if ( n == 1 )
    {
	host = (char *)lua_tostring( L, 1 );
    }


    /* creates a table/array to hold the results */
    lua_newtable(L);


    /*
     * Scan the host and find the open ports.
     */
    CPortScanner scanner( host );
    std::vector< int > openPorts = scanner.doScan();


    /*
     * For each open port add it to the table.
     */
    std::vector< int >::iterator i;
    int found = 0;

    for( i = openPorts.begin(); i != openPorts.end(); i++ )
    {
	/* The open port number detected. */
	int portNumber = *i;

	/* Store in the table. */
	lua_pushnumber(L, found);
	lua_pushnumber(L, portNumber );
	lua_settable(L, -3);

	/* Increase the number of open ports we found. */
	found++;
    }

    /* Make sure LUA knows how big our table is. */
    lua_pushliteral(L, "n");
    lua_pushnumber(L, found -1);
    lua_rawset(L, -3);

    /* return the number of results */
    return 1;
}


/**
 * Return the size of a file.
 */
/* static */ int CLUAInstance::GetFileSize( const char *fileName ) 
{

    FILE *file;

    file = fopen( fileName, "r" );
    if ( file == NULL )
    {
	return 0;
    }

    
    if ( fseek( file, 0, SEEK_END ) != 0 )
    {
	/* printf( "Error seeking" ); */
	fclose( file );
	return 0;
    }
 
    int pos = ftell( file );

    fclose( file );

  
    return pos;
}


/* static */ const char *CLUAInstance::readCompleteFile( const char *fileName )
{
    int size = GetFileSize( fileName );

    char *data = (char *)malloc( size + 1 );
    if ( data == NULL )
    {
	return( NULL );
    }

    memset( data, '\0', size + 1 );

    FILE *f = fopen( fileName, "r" );
    fread( data, 1, size, f );
    fclose( f );

    return( (const char *)data );
}


/*-------------------------------------------------------------------------*\
* Returns a string describing the last host manipulation error.
\*-------------------------------------------------------------------------*/
static char *host_strerror(void)
{
    switch (h_errno) {
        case HOST_NOT_FOUND: return "host not found";
        case NO_ADDRESS: return "unable to resolve host name";
        case NO_RECOVERY: return "name server error";
        case TRY_AGAIN: return "name server unavailable, try again later";
        default: return "unknown error";
    }
}

/**
 * C++ function to convert a hostname to an IP address.
 */
/* static */ int CLUAInstance::LUAToIP( lua_State *L )
{
    int n         = lua_gettop(L);
    if ( n != 1 && (!lua_isstring(L, 1) ) )
    {
        lua_pushstring(L, "'to_ip' requires a single string argument." );
	lua_error(L);
    }

    const char *address = luaL_check_string(L, 1);
    struct in_addr addr;
    struct hostent *hp;
    if (inet_aton(address, &addr))
        hp = gethostbyaddr((char *) &addr, sizeof(addr), AF_INET);
    else hp = gethostbyname(address);
    if (!hp) {
        lua_pushstring(L, host_strerror());
        return 1;
    }
    addr = *((struct in_addr *) hp->h_addr);
    lua_pushstring(L, inet_ntoa(addr));
    return 1;
}

/**
 * C++ function to convert an IP address to a hostname.
 */
/* static */ int CLUAInstance::LUAToHost( lua_State *L )
{
    int n         = lua_gettop(L);
    if ( n != 1 && (!lua_isstring(L, 1) ) )
    {
        lua_pushstring(L, "'to_host' requires a single string argument." );
	lua_error(L);
    }
    
    const char *address = lua_tostring(L,1);
    struct in_addr addr;
    struct hostent *hp;
    if (inet_aton(address, &addr)) 
        hp = gethostbyaddr((char *) &addr, sizeof(addr), AF_INET);
    else hp = gethostbyname(address);
    if (!hp) {
        lua_pushstring(L, host_strerror());
        return 1;
    }
    lua_pushstring(L, hp->h_name);
    return 1;

}
