/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/**
 *  luainstance.h
 *   Singleton wrapper around a LUA intepretter.
 *
 *  See:
 *     http://www.steve.org.uk/Software/lua-c++/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */



#ifndef _LUAINSTANCE_H
#define _LUAINSTANCE_H 1


#include <string>


/**
 * LUA Is C - protect ourself.
 */
extern "C" 
{
    #include <lua.h>
    #include <lauxlib.h>
    #include <lualib.h>
}




/**
 * This is the singleton accessor to a LUA intepretter object.
 *
 * This class both
 *
 * - Wraps the LUA instance into a single globally accessible object
 * - Allows LUA script functions to be called from a C++ application.
 *
 * Sample usage looks like this:
 *
 * @code
 *
 *  CLUAInstance *lua = CLUAInstance::getInstance();
 *  lua->runScript( "lua.lua" );
 *  lua->callFunction( "test", "si>i", "string parameter", 2, &ret );
 *  printf("Function returned: %d\n", ret );
 *
 * @endcode
 *
 *
 * @author Steve Kemp  http://www.steve.org.uk/
 * @version $Id: luainstance.h,v 1.8 2005/10/20 15:51:44 steve Exp $
 */
class CLUAInstance
{

public:
    /**
     * This class is an example of the Singleton Design Pattern, so there is
     * only one instance of this object ever in use.
     * @return The single CLUAInstance object.
     *
     */
    static CLUAInstance *getInstance();


    /**
     * Destructor.
     */
    ~CLUAInstance( );
    

    /**
     * Run the given script.
     * @param fileName The name of the LUA script to load and execute.
     */
    void runScript( std::string fileName );


    /**
     * Call a LUA function from C++.
     *
     * Here is an example of using this function to call a script
     * function:
     *
     * @code 
     *  callFunction("f", "dd>d", x, y, &z);
     * @endcode
     *
     * This calls the function "f" with two "double" arguments, returning a double.
     *
     * @param func The name of the function to call.
     * @param sig A format string of input and output options with a ">" between them.
     * @see callFunctionError
     * @author http://www.lua.org/pil/25.3.html
     */
    void callFunction(const char *func, const char *sig, ...);


    /**
     * This is an error-handling routine invoked if "callFunction" fails.
     * @param fmt The format string containing the error text.
     * @see callFunction
     */
    void callFunctionError( const char *fmt, ...);


protected:


    /**
     * Constructor - this is protected as this class uses
     * the singleton design pattern.
     */
    CLUAInstance( );


private:


    /**
     * Register the functions we make available to the scripts
     * executed by "runScript".
     * @see runScript
     */
    void registerFunctions();


    /**
     * The one and only CIntroScreen object.
     * @see getInstance
     */
    static CLUAInstance *m_instance;


    /**
     * The LUA intepretter object we use and wrap.
     */
    lua_State*  m_lua;


    /**
     * C function to implement the lua "getenv" function.
     */
    static  int LUAGetURI( lua_State *L );

    /**
     * C++ function to lookup a service by port number.
     */
    static  int LUAServiceByPort( lua_State *L );

    /**
     * C++ function to lookup a service by port number.
     */
    static  int LUAServiceByName( lua_State *L );


    /**
     * C++ function to detect a service, heuristically.
     */
    static int LUADetectService( lua_State *L );

    /**
     * C++ function to MD5 checksum a string
     */
    static int LUAMD5Sum( lua_State *L );

    /**
     * C++ function to MD5 a file's contents.
     */
    static int LUAMD5SumFile( lua_State *L );

    /**
     * C++ function to SHA1 checksum a string
     */
    static int LUASHA1Sum( lua_State *L );


    /**
     * C++ function to SHA1 checksum a file's contents.
     */
    static int LUASHA1SumFile( lua_State *L );


    /**
     * C++ function to return open ports upon a host..
     */
    static int LUAOpenPorts( lua_State *L );

    /**
     * C++ function to convert an IP address to a hostname.
     */
    static int CLUAInstance::LUAToHost( lua_State *L );

    /**
     * C++ function to convert a hostname to an IP address.
     */
    static int CLUAInstance::LUAToIP( lua_State *L );


    /**
     * Return the size of file.
     */
    static int GetFileSize( const char *fileName );

    /**
     * Return the contents of the given file, or NULL on error.
     */
    static const char *readCompleteFile( const char *fileName );

};


#endif /* _LUAINSTANCE_H */
