/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */

/**
 *  main.cpp
 *   Simple driver for Steve's Simple Security Scanner.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>


#include "getopt.h"
#include "main.h"
#include "luainstance.h"
#include "version.h"


/*
 * Options we accept.
 */
struct option options[] = {
  { "help",        no_argument,       NULL, 'h' },
  { "version",     no_argument,       NULL, 'v' },
  { "port",        required_argument, NULL, 'p' },
  { "target",      required_argument, NULL, 't' },
  { "script",      required_argument, NULL, 's' },
  {  0,            0,                 0,    0}
};





/**
 * Entry point to this program.
 * @param argc The count of parameters passed to this program by the OS.
 * @param argv An array containing the actual argument strings.
 * @return The status code we finish with to the OS.<
 */
int main( int argc, char *argv[] )
{
    char *host    = NULL;
    char *script  = NULL;
    int port      = 0;
    int opt_index = 0;
    int retval    = 0;
    
    retval = getopt_long (argc, argv, "hvp:s:t:", options, &opt_index);
    while (retval != EOF)
    {
	switch (retval)
        {
	case '?': /* FALL through. */
	case 'h': /* show usage */
	    showHelp( argv[0] );
	    exit( 0 );
            break;

	case 't':
	    host   = optarg;
            break;

	case 's':
	    script = optarg;
            break;

	case 'p':
	    port = atoi( optarg );
            break;

	case 'v': /* FALL through */
	    showVersions( argv[0] );
	    exit( 0 );
	    break;

	}

	
	retval = getopt_long (argc, argv, "hvp:s:t:", options, &opt_index);
    }


    /* If the user specified a test script then run it. */
    if ( script != NULL )
    {
	CLUAInstance *lua = CLUAInstance::getInstance();
	lua->runScript( std::string( SCRIPTS_DIR ) + std::string("/fortress.init" ) );	
	lua->runScript( script );
	lua->runScript( std::string( SCRIPTS_DIR ) + std::string("/fortress.destroy" ) );
	delete( lua );
	exit( 1 );
    }

    /* If we have a host then run the scripts. */
    if ( host != NULL )
    {
	runScripts( SCRIPTS_DIR, host, port );
	return( 0 );
    }
    else
    {
	/* Otherwise assume all further options, if any, are scripts
	 * to execute.
	 */
	if (optind < argc)
	{
	    CLUAInstance *lua = CLUAInstance::getInstance();
	    lua->runScript( std::string( SCRIPTS_DIR ) + std::string("/fortress.init" ) );	

	    /*
	     * Load and execute each referenced command line script.
	     */
	    while (optind < argc)
	    {
		lua->runScript( argv[ optind++] );

	    }

	    lua->runScript( std::string( SCRIPTS_DIR ) + std::string("/fortress.destroy" ) );
	    delete( lua );

	}
	else
	{
	    printf("You must specify either a hostname, or a script name\n" );
	    printf("\n");
	    showHelp( argv[0] );
	}
    }

    return( 0 );
}



/**
 * Load and execute each LUA script in the given directory.
 * We call the function 'testHost' upon each loaded script,
 * so that must sexist.
 * @param directory The directory to examine scripts within.
 * @param host The host we're scanning.
 * @param port The port number to investigate.
 * @return The number of scripts which executed.
 */
int runScripts( char *directory, char *host, int port )
{
    DIR *pDIR;
    struct dirent *pDirEnt;

    pDIR = opendir( directory );
  
    if ( pDIR == NULL ) 
    {
        return( 0 );
    }


    /* 
     * Run the startup script.
     */
    CLUAInstance *lua = CLUAInstance::getInstance();
    lua->runScript( directory + std::string("/fortress.init" ) );


    /* Get each directory entry from pDIR and print its name */
    pDirEnt = readdir( pDIR );
    for ( pDirEnt = readdir(pDIR); pDirEnt != NULL; pDirEnt = readdir(pDIR))
    {
        if ( ( strcmp( ".", pDirEnt->d_name  ) == 0  ) ||
	     ( strcmp( "..", pDirEnt->d_name ) == 0) )
	    continue;

	/*
	 * Get file suffix.
	 */
        char *suffix = strrchr( pDirEnt->d_name, '.' );
        if ( suffix != NULL ) 
        {
	    suffix++;

	    /*
	     * If we have a suffix, is it 'lua' ?
	     */
	    if ( strcmp( suffix, "lua" ) == 0 )
	    {
		int status;

	        CLUAInstance *lua = CLUAInstance::getInstance();
		lua->runScript( directory + std::string( pDirEnt->d_name ) );
		lua->callFunction( "test_host", "si>i", host, port, &status );
	    }
        }
    }


    /* 
     * Run the finished script.
     */
    lua->runScript( directory + std::string("/fortress.destroy" ) );
    
    /* Release the open directory */
    closedir( pDIR );
    return 1;
}



/**
 * Show the version of this program, and the CVS revision information
 * of all included source code.
 * @param program The name of this program, from argv[0]
 */
void showVersions( char *program )
{
    printf( "%s - version %s by Steve Kemp\n", program, VERSION );

    int count  = 0;
    int length = strlen( program );

    while( count < ( length + 3 ) )
    {
	printf( " " );
	count ++;
    }
    printf( "http://www.steve.org.uk/\n");
    printf( "\nSource Versions:\n" );

    std::vector< CVersionRecord *> *versions;
    versions = CVersion::GetInstance()->GetVersionRecords();
        
    std::vector< CVersionRecord *>::iterator i;
    
    for( i = versions->begin(); i != versions->end(); i++ )
    {
	CVersionRecord *v = *i;
	printf("\t%s \t %s\n",
	       v->GetVersion().c_str(),
	       v->GetFile().c_str() );
    }

    printf( "\nScripts directory: %s\n", SCRIPTS_DIR );
}



/**
 * Show brief usage information for this binary.
 * @param program The name of this program, from argv[0]
 */
void showHelp( char *program )
{
    printf( "%s - version %s by Steve Kemp\n", program, VERSION );

    int count  = 0;
    int length = strlen( program );

    while( count < ( length + 3 ) )
    {
	printf( " " );
	count ++;
    }
    printf( "http://www.steve.org.uk/\n");


    printf("\nUsage:\n");
    printf(" [ %s scriptName.lua | %s option1 option2 ]\n\n",program, program );
    printf("Options:\n");
    printf("  --help      - Show this help\n" );
    printf("  --port xx   - Set port to scan\n" );
    printf("  --script xx - Specify a script to run\n" );
    printf("  --target=xx - Set host to scan\n" );
    printf("  --version - Show version of main binary, and all source\n" );

}
