/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */


/**
 *  version.cc
 *   A painless way of keeping track of source versions.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */

/*
 * I'm quite proud of this code, if you have any feedback I'd be intersted
 * in hearing it.
 */


#include <algorithm>  /* Needed for 'sort' */

#include "version.h"


/**
 * The CVersion singleton object.
 */
CVersion * CVersion::m_instance = NULL;


/**
 *  Dummy object which will record the name and version of this file.
 */
InstallVersion dummyVersion( __FILE__, __DATE__, "$Revision: 1.2 $" );




/**
 * Create a new version information record.
 * @param fileName The file that this record covers
 * @param versionString A version string for this file.
 */
CVersionRecord::CVersionRecord( char *fileName, char *date, char *versionString )
{
    m_fileName = fileName;
    m_date     = date;
    m_version  = versionString;
}

/**
 * Destructor
 */
CVersionRecord::~CVersionRecord()
{
}

/**
 * Get the filename that was stored in this record.
 * @return The filename that was stored in this record.
 */
std::string CVersionRecord::GetFile( ) 
{
    return( m_fileName ); 
}

/**
 * Get the version string that was stored in this record.
 * @return The version string that was stored in this record.
 */
std::string CVersionRecord::GetVersion( ) 
{ 
    return( m_version ); 
}

/**
 * Get the compilation date and time of this file.
 * @return The date and time at which the file was last recompiled.
 */
std::string CVersionRecord::GetDate( )
{
    return( m_date );
}


/**
 * Constructor.
 */
CVersion::CVersion()
{
    m_versions = new std::vector< CVersionRecord * >;
}

/**
 * Destructor.
 */
CVersion::~CVersion()
{
    delete( m_versions );
}



/**
 * This class is implemented as a singleton pattern, this method
 * allows access to the single, global, version object.
 * @return A pointer to this single version interface.
 */
/* static */ CVersion * CVersion::GetInstance()
{
    if ( m_instance == NULL )
    {
	m_instance = new CVersion();
    }

    return( m_instance );
};

/**
 * Add a version record to this object.
 * @param file The filename that we're referring to.
 * @param version The version ID of that file.
 */
void CVersion::AddFileVersion( char *file, char *date, char *version )
{
    CVersionRecord *rec = new CVersionRecord( file, date, version );
    m_versions->push_back( rec );
}


/**
 * Retrieve the current collection of version records.
 * @return A vector containing the version records.
 */
std::vector< CVersionRecord *> *CVersion::GetVersionRecords()
{
    /**
     * Sort the records before returning them.
     */
    sort( m_versions->begin(), m_versions->end(), CVersion::compareFunction );
    return( m_versions );
}

/**
 * Compare two version records - this is static so that
 * users may request a sort, without any effort.
 *
 * @param a The first version record to compare.
 * @param b The second version record to compare.
 * @return 1 if a < b, 0 otherwise.
 */
/*static*/ int CVersion::compareFunction( CVersionRecord *a, CVersionRecord *b )
{
    std::string aF = a->GetFile();
    std::string bF = b->GetFile();

    return( aF < bF );
}
