/* -*-mode: C++; style: K&R; c-basic-offset: 4 ; -*- */


/**
 *  version.h 
 *   A painless way of keeping track of source versions.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *  Steve Kemp
 *  ---
 *  http://www.steve.org.uk/
 *
 */

/*
 * I'm quite proud of this code, if you have any feedback I'd be intersted
 * in hearing it.
 */


#ifndef _CVERSION_H_
#define _CVERSION_H_ 1

#include <vector>
#include <string>


/**
 *   This is a version holding record, which stores the version string of
 * a given source file.
 *
 *   When this versioning system is used throughout a program each
 * source file will have an associated CVersionRecord object, this information
 * can be queried or displayed, etc, at runtime.
 *
 *   When the program wishes to query its own version information
 * it can use the CVersion class to return a list of all the individual
 * CVersionRecord objects - then display, or use that list.
 *
 * @see CVersion
 * @author Steve Kemp http://www.steve.org.uk/
 * @version $Revision: 1.2 $
 */
class CVersionRecord
{
public:
    /**
     * Create a new version information record.
     * @param fileName The file that this record covers, use _ _ FILE _ _
     * @param date The current date, use _ _ DATE _ _
     * @param versionString A version string for this file.
     */
    CVersionRecord( char *fileName, char *date, char *versionString );

    /**
     * Destructor.
     */
    virtual ~CVersionRecord();
 
public:
    /**
     * Get the filename that was stored in this record.
     * @return The filename that was stored in this record.
     */
    std::string GetFile();
  
    /**
     * Get the version string that was stored in this record.
     * @return The version string that was stored in this record.
     */
    std::string GetVersion();

    /**
     * Get the compilation date and time of this file.
     * @return The date and time at which the file was last recompiled.
     */
    std::string GetDate();

private:
    /**
     * The filename this record represents.
     */
    std::string m_fileName;
    
    /**
     * The version string stored in this record.
     */
    std::string m_version;

    /**
     * The date and time this file was compiled.
     */
    std::string m_date;

};


/**
 *  A class for maintaining version records of all the source files which
 * make up a binary.
 *
 *  This class is basically a means of storing a collection of
 * CVersionRecord objects.
 *
 *  The intention is that each C++ class in a project will have a
 * single CVersionRecord object associated with it - this class will
 * be the single, global, store for these records.
 *
 *  This class is a singleton, which allows you to access the stored
 * records, for display or other uses.
 *
 * @see CVersionRecord
 * @author Steve Kemp http://www.steve.org.uk/
 * @version $Revision: 1.2 $
 */
class CVersion
{
public:
    /**
     * This class is implemented as a singleton pattern, this method
     * allows access to the single, global, version object.
     * @return A pointer to this single version interface.
     */
    static CVersion * GetInstance();

    /**
     * Destructor.
     */
    virtual ~CVersion();

    /**
     * Add a version record to this object.
     * @param file The filename that we're referring to.
     * @param date The date this file was compiled.
     * @param version The version ID of that file.
     */
    void AddFileVersion( char *file, char *date, char *version );

    /**
     * Retrieve the current collection of version records.
     * @return A vector containing the version records.
     */
    std::vector< CVersionRecord *> * GetVersionRecords();

    /**
     * Compare two version records - this is static so that
     * users may request a sort without any effort.
     *
     * @param a The first version record to compare.
     * @param b The second version record to compare.
     * @return 1 if a < b, 0 otherwise.
     */
    static int compareFunction( CVersionRecord *a, CVersionRecord *b );  

protected:
    /**
     * Protected constructor.  (This is a singleton).
     */
    CVersion();

    /**
     * The single instance of this class.
     */
    static CVersion *m_instance;
  
private:
    /**
     * The collection of version records that we're storing.
     */
    std::vector< CVersionRecord * > *m_versions;

private:

    /**
     * Not assignable this is a singleton object.
     */
    void operator=(CVersion &src );

    /**
     * Not copyable this is a singleton object.
     */
    CVersion( CVersion &src );

};


/**
 * This is a convienence class, which allows any C++ program to
 * easily maintain a list of all the files + version numbers in
 * it.
 *
 *  Usage is extremely simple, in each C++ file include the
 * following two lines, near the top:
 *
 * @code
 *
 * #include "version.h"
 * InstallVersion dummy( __ FILE __, __ DATE __, "$Revision: 1.2 $" );
 *
 * @endcode
 *
 *  This will install the current file into the versioning
 * system, which you may then query, like this:
 *
 * @code
 *      printf( "\nSource Versions:\n" );
 *      std::vector< CVersionRecord *> *versions;
 *      versions = CVersion::GetInstance()->GetVersionRecords();
 *	
 *      std::vector< CVersionRecord *>::iterator i;
 *
 *      for( i = versions->begin(); i != versions->end(); i++ )
 *      {
 *          CVersionRecord *v = *i;
 *          printf("\t%s \t %s\n",
 *                 v->GetVersion(),
 *                 v->GetFile() );
 *      }
 *
 * @endcode
 *
 * @see CVersion CVersionRecord
 * @author Steve Kemp http://www.steve.org.uk/
 * @version $Revision: 1.2 $
 *
 */
class InstallVersion
{
public:
    /**
     * Constructor.
     * The sole purpose of this class, and its constructor
     * is to provide a simple means of instantiating an
     * instance of the CVersion class - then adding information
     * about the current file to it.
     *
     * This makes manual addition needless, and makes this
     * versioning system simple to use.
     * 
     * @param file The name of the current source file, use _ _FILE_ _
     * @param date The current date, use _ _ DATE _ _
     * @param version The version to go along with this string, use $Revision: 1.2 $
     */
    InstallVersion( char *file, char *date, char *version )
    {
	CVersion *vHolder = CVersion::GetInstance();
	vHolder->AddFileVersion( file, date, version );
    };

};




#endif /* _CVERSION_H_ */

